from rest_framework import serializers

from webkassa.models import Check


class CheckSerializer(serializers.ModelSerializer):
    CheckNumber = serializers.CharField(source='check_number', required=False)
    DateTime = serializers.CharField(source='date_time', required=False)
    Cashbox = serializers.JSONField(source='cash_box', required=False)
    CheckOrderNumber = serializers.IntegerField(source='check_order_number', required=False)
    ShiftNumber = serializers.IntegerField(source='shift_number', required=False)
    EmployeeName = serializers.CharField(source='employee_name', required=False)
    TicketUrl = serializers.URLField(source='ticket_url', required=False)
    TicketPrintUrl = serializers.URLField(source='ticket_print_url', required=False)
    ExternalCheckNumber = serializers.SerializerMethodField(read_only=True)

    class Meta:
        model = Check
        fields = ('CheckNumber', 'DateTime', 'Cashbox',
                  'CheckOrderNumber', 'ShiftNumber', 'EmployeeName', 'TicketUrl', 'TicketPrintUrl',
                  'ExternalCheckNumber')
        read_only_fields = ('external_check_number',)

    def get_ExternalCheckNumber(self, obj):
        return str(obj.id)



from cryptography.fernet import Fernet

from django.conf import settings


def encrypt_password(raw_password):
    fernet = Fernet(settings.WEBKASSA_CONFIG['encryption_key'])
    return fernet.encrypt(raw_password.encode()).decode()


def decrypt_password(password):
    fernet = Fernet(settings.WEBKASSA_CONFIG['encryption_key'])
    return fernet.decrypt(bytes(password, 'ascii')).decode()



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WebkassaConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'webkassa'
    verbose_name = _('Webkassa')



from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from webkassa.models import Check, WebKassaAccount, WebkassaErrorLog


@admin.register(Check)
class CheckAdmin(admin.ModelAdmin):
    list_display = ('id', 'date_time', 'show_ticket_url', 'check_order_number')
    list_display_links = ('id', 'date_time', 'check_order_number')

    def show_ticket_url(self, obj):
        return format_html("<a href='{url}'>{url}</a>", url=obj.ticket_url)

    show_ticket_url.short_description = _('Ticket URL')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False


@admin.register(WebKassaAccount)
class WebKassaAccountAdmin(admin.ModelAdmin):
    list_display = ('id', 'email', 'cashbox_unique_number', 'updated_at')
    list_display_links = ('id', 'email', 'cashbox_unique_number', 'updated_at')

    readonly_fields = ('id', 'created_at', 'updated_at', 'is_encrypted', 'token')


@admin.register(WebkassaErrorLog)
class WebkassaErrorLogAdmin(admin.ModelAdmin):
    list_display = ('check', 'code', 'text', 'created_at')
    list_display_links = ('check', 'code', 'text', 'created_at')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('check_obj')



# Generated by Django 4.2.3 on 2023-07-18 06:55

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Check',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('ticket_url', models.URLField(blank=True, null=True, verbose_name='Ticket URL')),
                ('ticket_print_url', models.URLField(blank=True, null=True, verbose_name='Ticket URL for print')),
                ('check_number', models.CharField(blank=True, max_length=255, null=True, verbose_name='Check number')),
                ('date_time', models.CharField(blank=True, max_length=255, null=True, verbose_name='Check datetime')),
                ('cash_box', models.JSONField(blank=True, null=True, verbose_name='Cashbox')),
                ('check_order_number', models.BigIntegerField(default=0, verbose_name='Check order number')),
                ('shift_number', models.BigIntegerField(default=0, verbose_name='Shift number')),
                ('employee_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Employee name')),
            ],
            options={
                'verbose_name': 'Check',
                'verbose_name_plural': 'Checks',
            },
        ),
        migrations.CreateModel(
            name='WebKassaAccount',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('password', models.CharField(max_length=300)),
                ('cashbox_unique_number', models.CharField(max_length=255)),
                ('is_encrypted', models.BooleanField(default=False)),
                ('token', models.CharField(default='', max_length=100)),
            ],
            options={
                'verbose_name': 'Webkassa account',
                'verbose_name_plural': 'Webkassa accounts',
            },
        ),
        migrations.CreateModel(
            name='WebkassaErrorLog',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('code', models.CharField(max_length=10)),
                ('text', models.TextField()),
                ('check_obj', models.ForeignKey(blank=True, null=True, on_delete=models.deletion.CASCADE,
                                                related_name='error_logs', to='webkassa.check', verbose_name='Check')),
            ],
            options={
                'verbose_name': 'Webkassa error',
                'verbose_name_plural': 'Webkassa errors',
            },
        ),
    ]
